@@ -49,7 +49,7 @@ except:
 ##!/bin/sh
 #LANG=$(for i in *.po; do  j=${i/.po/}; echo -n "_('"$j"')":" '"$j"', " ; done)
 #echo "{_('en'):'en'",$LANG"}"
-langs = {_('English'): 'en', _('Bulgarian'): 'bg', _('Briton'): 'br', _('Czech'): 'cs', _('German'): 'de', _('Greek'): 'el', _('Esperanto'): 'eo', _('Spanish'): 'es', _('Basc'): 'eu', _('French'): 'fr', _('Croatian'): 'hr', _('Italian'): 'it', _('Norvegian b'): 'nb', _('Dutch'): 'nl', _('Norvegian'): 'no', _('Polish'): 'pl', _('Portuguese'): 'pt', _('Brazilian Portuguese'): 'pt_BR', _('Russian'): 'ru', _('Slovak'): 'sk', _('Swedish'): 'sv', _('Chinese (Ch)'): 'zh_CN'}
+langs = {_('English'): 'en', _('Belarusian'): 'be', _('Bulgarian'): 'bg', _('Briton'): 'br', _('Czech'): 'cs', _('German'): 'de', _('Greek'): 'el', _('British'): 'en_GB', _('Esperanto'): 'eo', _('Spanish'): 'es', _('Basc'): 'eu', _('French'): 'fr', _('Croatian'): 'hr', _('Italian'): 'it', _('Norwegian (b)'): 'nb', _('Dutch'): 'nl', _('Norwegian'): 'no', _('Polish'): 'pl', _('Portuguese'): 'pt', _('Brazilian Portuguese'): 'pt_BR', _('Russian'): 'ru', _('Serbian'): 'sr', _('Slovak'): 'sk', _('Swedish'): 'sv', _('Chinese (Ch)'): 'zh_CN'}
 
 
 ################################################################################
@@ -91,15 +91,22 @@ class ChatControlBase(MessageControl):
 			type_]))
 
 	def draw_banner(self):
-		self._paint_banner()
+		'''Draw the fat line at the top of the window that 
+		houses the icon, jid, ... 
+		'''
+		self.draw_banner_text()
 		self._update_banner_state_image()
-		# Derived types SHOULD implement this
+		# Derived types MAY implement this
+
+	def draw_banner_text(self):
+		pass # Derived types SHOULD implement this
 
 	def update_ui(self):
 		self.draw_banner()
 		# Derived types SHOULD implement this
 
 	def repaint_themed_widgets(self):
+		self._paint_banner()
 		self.draw_banner()
 		# Derived classes MAY implement this
 
@@ -110,10 +117,10 @@ class ChatControlBase(MessageControl):
 	event_keymod):
 		pass # Derived should implement this rather than connecting to the event itself.
 
-	def __init__(self, type_id, parent_win, widget_name, display_names, contact,
-	acct, resource = None):
+	def __init__(self, type_id, parent_win, widget_name, contact, acct, 
+	resource = None):
 		MessageControl.__init__(self, type_id, parent_win, widget_name,
-			display_names,	contact, acct, resource = resource);
+			contact, acct, resource = resource);
 		# when/if we do XHTML we will put formatting buttons back
 		widget = self.xml.get_widget('emoticons_button')
 		id = widget.connect('clicked', self.on_emoticons_button_clicked)
@@ -123,6 +130,7 @@ class ChatControlBase(MessageControl):
 		self.handlers[id] = self.widget
 
 		widget = self.xml.get_widget('banner_eventbox')
+		widget.set_property('height-request', gajim.config.get('chat_avatar_height'))
 		id = widget.connect('button-press-event',
 			self._on_banner_eventbox_button_press_event)
 		self.handlers[id] = widget
@@ -137,6 +145,11 @@ class ChatControlBase(MessageControl):
 		id = widget.connect('value-changed',
 			self.on_conversation_vadjustment_value_changed)
 		self.handlers[id] = widget
+		id = widget.connect('changed',
+			self.on_conversation_vadjustment_changed)
+		self.handlers[id] = widget
+		self.scroll_to_end_id = None
+		self.was_at_the_end = True
 		# add MessageTextView to UI and connect signals
 		self.msg_scrolledwindow = self.xml.get_widget('message_scrolledwindow')
 		self.msg_textview = MessageTextView()
@@ -163,8 +176,7 @@ class ChatControlBase(MessageControl):
 		# the following vars are used to keep history of user's messages
 		self.sent_history = []
 		self.sent_history_pos = 0
-		self.typing_new = False
-		self.orig_msg = ''
+		self.orig_msg = None
 
 		# Emoticons menu
 		# set image no matter if user wants at this time emoticons or not
@@ -208,6 +220,7 @@ class ChatControlBase(MessageControl):
 
 		self.style_event_id = 0
 		self.conv_textview.tv.show()
+		self._paint_banner()
 
 		# For JEP-0172
 		self.user_nick = None
@@ -497,7 +510,7 @@ class ChatControlBase(MessageControl):
 		return False
 
 	def send_message(self, message, keyID = '', type = 'chat', chatstate = None,
-	msg_id = None, composing_jep = None, resource = None):
+	msg_id = None, composing_xep = None, resource = None):
 		'''Send the given message to the active tab. Doesn't return None if error
 		'''
 		if not message or message == '\n':
@@ -507,7 +520,7 @@ class ChatControlBase(MessageControl):
 		if not self._process_command(message):
 			ret = MessageControl.send_message(self, message, keyID, type = type,
 				chatstate = chatstate, msg_id = msg_id,
-				composing_jep = composing_jep, resource = resource,
+				composing_xep = composing_xep, resource = resource,
 				user_nick = self.user_nick)
 			if ret:
 				return ret
@@ -533,9 +546,7 @@ class ChatControlBase(MessageControl):
 		else:
 			self.sent_history.append(message)
 			self.sent_history_pos = size + 1
-
-		self.typing_new = True
-		self.orig_msg = ''
+		self.orig_msg = None
 
 	def print_conversation_line(self, text, kind, name, tim,
 		other_tags_for_name = [], other_tags_for_time = [], 
@@ -546,7 +557,7 @@ class ChatControlBase(MessageControl):
 		full_jid = self.get_full_jid()
 		textview = self.conv_textview
 		end = False
-		if textview.at_the_end() or kind == 'outgoing':
+		if self.was_at_the_end or kind == 'outgoing':
 			end = True
 		textview.print_conversation_line(text, jid, kind, name, tim,
 			other_tags_for_name, other_tags_for_time, other_tags_for_text,
@@ -652,7 +663,7 @@ class ChatControlBase(MessageControl):
 	def set_control_active(self, state):
 		if state:
 			jid = self.contact.jid
-			if self.conv_textview.at_the_end():
+			if self.was_at_the_end:
 				# we are at the end
 				type_ = 'printed_' + self.type_id
 				if self.type_id == message_control.TYPE_GC:
@@ -667,18 +678,23 @@ class ChatControlBase(MessageControl):
 
 	def bring_scroll_to_end(self, textview, diff_y = 0):
 		''' scrolls to the end of textview if end is not visible '''
+		if self.scroll_to_end_id:
+			# a scroll is already planned
+			return
 		buffer = textview.get_buffer()
 		end_iter = buffer.get_end_iter()
 		end_rect = textview.get_iter_location(end_iter)
 		visible_rect = textview.get_visible_rect()
 		# scroll only if expected end is not visible
 		if end_rect.y >= (visible_rect.y + visible_rect.height + diff_y):
-			gobject.idle_add(self.scroll_to_end_iter, textview)
+			self.scroll_to_end_id = gobject.idle_add(self.scroll_to_end_iter,
+				textview)
 
 	def scroll_to_end_iter(self, textview):
 		buffer = textview.get_buffer()
 		end_iter = buffer.get_end_iter()
 		textview.scroll_to_iter(end_iter, 0, False, 1, 1)
+		self.scroll_to_end_id = None
 		return False
 
 	def size_request(self, msg_textview , requisition):
@@ -709,11 +725,15 @@ class ChatControlBase(MessageControl):
 		if diff_y != 0:
 			if conversation_height + diff_y < min_height:
 				if message_height + conversation_height - min_height > min_height:
-					self.msg_scrolledwindow.set_property('vscrollbar-policy', 
-						gtk.POLICY_AUTOMATIC)
-					self.msg_scrolledwindow.set_property('height-request', 
-						message_height + conversation_height - min_height)
-					self.bring_scroll_to_end(msg_textview)
+					policy = self.msg_scrolledwindow.get_property(
+						'vscrollbar-policy')
+					# scroll only when scrollbar appear
+					if policy != gtk.POLICY_AUTOMATIC:
+						self.msg_scrolledwindow.set_property('vscrollbar-policy', 
+							gtk.POLICY_AUTOMATIC)
+						self.msg_scrolledwindow.set_property('height-request', 
+							message_height + conversation_height - min_height)
+						self.bring_scroll_to_end(msg_textview)
 			else:
 				self.msg_scrolledwindow.set_property('vscrollbar-policy', 
 					gtk.POLICY_NEVER)
@@ -732,7 +752,15 @@ class ChatControlBase(MessageControl):
 
 		return True
 
-	def on_conversation_vadjustment_value_changed(self, widget):
+	def on_conversation_vadjustment_changed(self, adjustment):
+		# used to stay at the end of the textview when we shrink conversation
+		# textview.
+		if self.was_at_the_end:
+			self.conv_textview.bring_scroll_to_end(-18)
+		self.was_at_the_end = (adjustment.upper - adjustment.value - adjustment.page_size) < 18
+
+	def on_conversation_vadjustment_value_changed(self, adjustment):
+		self.was_at_the_end = (adjustment.upper - adjustment.value - adjustment.page_size) < 18
 		if self.resource:
 			jid = self.contact.get_full_jid()
 		else:
@@ -744,12 +772,9 @@ class ChatControlBase(MessageControl):
 		type_])):
 			return
 		if self.conv_textview.at_the_end() and \
-				self.parent_win.get_active_control() == self and \
-				self.parent_win.window.is_active():
+		self.parent_win.get_active_control() == self and \
+		self.parent_win.window.is_active():
 			# we are at the end
-			type_ = self.type_id
-			if type_ == message_control.TYPE_GC:
-				type_ = 'gc_msg'
 			if not gajim.events.remove_events(self.account, self.get_full_jid(),
 			types = ['printed_' + type_, type_]):
 				# There were events to remove
@@ -774,14 +799,13 @@ class ChatControlBase(MessageControl):
 
 	def sent_messages_scroll(self, direction, conv_buf):
 		size = len(self.sent_history) 
-		if self.typing_new:
-			#user was typing something and then went into history, so save
-			#whatever is already typed
+		if self.orig_msg is None:
+			# user was typing something and then went into history, so save
+			# whatever is already typed
 			start_iter = conv_buf.get_start_iter()
 			end_iter = conv_buf.get_end_iter()
 			self.orig_msg = conv_buf.get_text(start_iter, end_iter, 0).decode(
 				'utf-8')
-			self.typing_new = False
 		if direction == 'up':
 			if self.sent_history_pos == 0:
 				return
@@ -790,7 +814,7 @@ class ChatControlBase(MessageControl):
 		elif direction == 'down':
 			if self.sent_history_pos >= size - 1:
 				conv_buf.set_text(self.orig_msg);
-				self.typing_new = True
+				self.orig_msg = None
 				self.sent_history_pos = size
 				return
 
@@ -841,7 +865,7 @@ class ChatControl(ChatControlBase):
 	
 	def __init__(self, parent_win, contact, acct, resource = None):
 		ChatControlBase.__init__(self, self.TYPE_ID, parent_win,
-			'chat_child_vbox', (_('Chat'), _('Chats')), contact, acct, resource)
+			'chat_child_vbox', contact, acct, resource)
 			
 		# for muc use:
 		# widget = self.xml.get_widget('muc_window_actions_button')
@@ -849,9 +873,9 @@ class ChatControl(ChatControlBase):
 		id = widget.connect('clicked', self.on_actions_button_clicked)
 		self.handlers[id] = widget
 
-		self.hide_chat_buttons_always = gajim.config.get(
+		hide_chat_buttons_always = gajim.config.get(
 			'always_hide_chat_buttons')
-		self.chat_buttons_set_visible(self.hide_chat_buttons_always)
+		self.chat_buttons_set_visible(hide_chat_buttons_always)
 		self.widget_set_visible(self.xml.get_widget('banner_eventbox'),
 			gajim.config.get('hide_chat_banner'))
 		# Initialize drag-n-drop
@@ -1011,18 +1035,16 @@ class ChatControl(ChatControlBase):
 
 		self._update_gpg()
 
-	def draw_banner(self, chatstate = None):
-		'''Draw the fat line at the top of the window that 
-		houses the status icon, name, jid.  The chatstate arg should
-		only be used if the control's chatstate member is NOT to be use, such as
-		composing, paused, etc.
+	def draw_banner_text(self):
+		'''Draw the text in the fat line at the top of the window that 
+		houses the name, jid. 
 		'''
-		ChatControlBase.draw_banner(self)
-
 		contact = self.contact
 		jid = contact.jid
 
 		banner_name_label = self.xml.get_widget('banner_name_label')
+		banner_eventbox = self.xml.get_widget('banner_eventbox')
+		
 		name = contact.get_shown_name()
 		if self.resource:
 			name += '/' + self.resource
@@ -1049,8 +1071,10 @@ class ChatControl(ChatControlBase):
 
 		status = contact.status
 		if status is not None:
+			self.status_tooltip.set_tip(banner_eventbox, status)
+			self.status_tooltip.enable()
 			banner_name_label.set_ellipsize(pango.ELLIPSIZE_END)
-			status = helpers.reduce_chars_newlines(status, max_lines = 2)
+			status = helpers.reduce_chars_newlines(status, max_lines = 1)
 		status_escaped = gtkgui_helpers.escape_for_pango_markup(status)
 
 		font_attrs, font_attrs_small = self.get_font_attrs()
@@ -1059,18 +1083,19 @@ class ChatControl(ChatControlBase):
 		if cs and st in ('composing_only', 'all'):
 			if contact.show == 'offline':
 				chatstate = ''
-			elif contact.composing_jep == 'JEP-0085':
+			elif contact.composing_xep == 'XEP-0085':
 				if st == 'all' or cs == 'composing':
 					chatstate = helpers.get_uf_chatstate(cs)
 				else:
 					chatstate = ''
-			elif contact.composing_jep == 'JEP-0022':
+			elif contact.composing_xep == 'XEP-0022':
 				if cs in ('composing', 'paused'):
 					# only print composing, paused
 					chatstate = helpers.get_uf_chatstate(cs)
 				else:
 					chatstate = ''
-			elif chatstate is None:
+			else:
+				# When does that happen ? See [7797] and [7804]
 				chatstate = helpers.get_uf_chatstate(cs)
 
 			label_text = '<span %s>%s</span><span %s>%s %s</span>' % \
@@ -1082,9 +1107,6 @@ class ChatControl(ChatControlBase):
 		if status_escaped:
 			label_text += '\n<span %s>%s</span>' %\
 				(font_attrs_small, status_escaped)
-			banner_eventbox = self.xml.get_widget('banner_eventbox')
-			self.status_tooltip.set_tip(banner_eventbox, status)
-			self.status_tooltip.enable()
 		else:
 			self.status_tooltip.disable()
 		# setup the label that holds name and jid
@@ -1138,29 +1160,25 @@ class ChatControl(ChatControlBase):
 
 		chatstates_on = gajim.config.get('outgoing_chat_state_notifications') != \
 			'disabled'
-		composing_jep = contact.composing_jep
+		composing_xep = contact.composing_xep
 		chatstate_to_send = None
 		if chatstates_on and contact is not None:
-			if composing_jep is None:
+			if composing_xep is None:
 				# no info about peer
 				# send active to discover chat state capabilities
 				# this is here (and not in send_chatstate)
 				# because we want it sent with REAL message
 				# (not standlone) eg. one that has body
-				
-				#FIXME:
-				# Enable 3 next lines after 0.11 release.
-				# Having this disabled violate xep85 5.1.2 but then we don't break
-				# notifications between 0.10.1 and 0.11 See #2637
-				# if contact.our_chatstate:
-				#	# We already ask for xep 85, don't ask it twice
-				#	composing_jep = 'asked_once'
+
+				if contact.our_chatstate:
+					# We already asked for xep 85, don't ask it twice
+					composing_xep = 'asked_once'
 
 				chatstate_to_send = 'active'
 				contact.our_chatstate = 'ask' # pseudo state
 			# if peer supports jep85 and we are not 'ask', send 'active'
 			# NOTE: first active and 'ask' is set in gajim.py
-			elif composing_jep is not False:
+			elif composing_xep is not False:
 				#send active chatstate on every message (as JEP says)
 				chatstate_to_send = 'active'
 				contact.our_chatstate = 'active'
@@ -1170,7 +1188,7 @@ class ChatControl(ChatControlBase):
 				self._schedule_activity_timers()
 				
 		if not ChatControlBase.send_message(self, message, keyID, type = 'chat',
-		chatstate = chatstate_to_send, composing_jep = composing_jep):
+		chatstate = chatstate_to_send, composing_xep = composing_xep):
 			self.print_conversation(message, self.contact.jid,
 				encrypted = encrypted)
 
@@ -1297,6 +1315,8 @@ class ChatControl(ChatControlBase):
 		# Draw tab label using chatstate 
 		theme = gajim.config.get('roster_theme')
 		color = None
+		if not chatstate:
+			chatstate = self.contact.chatstate
 		if chatstate is not None:
 			if chatstate == 'composing':
 				color = gajim.config.get_per('themes', theme,
@@ -1318,7 +1338,7 @@ class ChatControl(ChatControlBase):
 			if chatstate in ('inactive', 'gone') and\
 			self.parent_win.get_active_control() != self:
 				color = self.lighten_color(color)
-		elif chatstate == 'active' : # active, get color from gtk
+		else: # active or not chatstate, get color from gtk
 			color = self.parent_win.notebook.style.fg[gtk.STATE_ACTIVE]
 		
 
@@ -1461,7 +1481,7 @@ class ChatControl(ChatControlBase):
 		if contact.show == 'offline':
 			return
 
-		if contact.composing_jep is False: # jid cannot do jep85 nor jep22
+		if contact.composing_xep is False: # jid cannot do xep85 nor xep22
 			return
 
 		# if the new state we wanna send (state) equals 
@@ -1469,7 +1489,7 @@ class ChatControl(ChatControlBase):
 		if contact.our_chatstate == state:
 			return
 
-		if contact.composing_jep is None:
+		if contact.composing_xep is None:
 			# we don't know anything about jid, so return
 			# NOTE:
 			# send 'active', set current state to 'ask' and return is done
@@ -1483,7 +1503,7 @@ class ChatControl(ChatControlBase):
 
 		# in JEP22, when we already sent stop composing
 		# notification on paused, don't resend it
-		if contact.composing_jep == 'JEP-0022' and \
+		if contact.composing_xep == 'XEP-0022' and \
 		contact.our_chatstate in ('paused', 'active', 'inactive') and \
 		state is not 'composing': # not composing == in (active, inactive, gone)
 			contact.our_chatstate = 'active'
@@ -1505,7 +1525,7 @@ class ChatControl(ChatControlBase):
 			self.reset_kbd_mouse_timeout_vars()
 
 		MessageControl.send_message(self, None, chatstate = state,
-			msg_id = contact.msg_id, composing_jep = contact.composing_jep)
+			msg_id = contact.msg_id, composing_xep = contact.composing_xep)
 		contact.our_chatstate = state
 		if contact.our_chatstate == 'active':
 			self.reset_kbd_mouse_timeout_vars()
@@ -1551,7 +1571,7 @@ class ChatControl(ChatControlBase):
 
 	def handle_incoming_chatstate(self):
 		''' handle incoming chatstate that jid SENT TO us '''
-		self.draw_banner()
+		self.draw_banner_text()
 		# update chatstate in tab for this chat
 		self.parent_win.redraw_tab(self, self.contact.chatstate)
 
@@ -1568,24 +1588,34 @@ class ChatControl(ChatControlBase):
 		if not gajim.config.get('show_avatar_in_chat'):
 			return
 
-		jid = self.contact.jid
-		jid_with_resource = jid
-		if resource:
-			jid_with_resource += '/' + resource
+		is_fake = False
+		if self.TYPE_ID == message_control.TYPE_PM:
+			is_fake = True
+			jid_with_resource = self.contact.jid # fake jid
+		else:
+			jid_with_resource = self.contact.jid
+			if resource:
+				jid_with_resource += '/' + resource
 
 		# we assume contact has no avatar
 		scaled_pixbuf = None
 
-		pixbuf = None
-		is_fake = False
-		if gajim.contacts.is_pm_from_jid(self.account, jid):
-			is_fake = True
 		pixbuf = gtkgui_helpers.get_avatar_pixbuf_from_cache(jid_with_resource,
 			is_fake)
 		if pixbuf == 'ask':
 			# we don't have the vcard
-			gajim.connections[self.account].request_vcard(jid_with_resource,
-				is_fake)
+			if self.TYPE_ID == message_control.TYPE_PM:
+				if self.gc_contact.jid:
+					# We know the real jid of this contact
+					real_jid = self.gc_contact.jid
+					if self.gc_contact.resource:
+						real_jid += '/' + self.gc_contact.resource
+				else:
+					real_jid = jid_with_resource
+				gajim.connections[self.account].request_vcard(real_jid,
+					jid_with_resource)
+			else:
+				gajim.connections[self.account].request_vcard(jid_with_resource)
 			return
 		if pixbuf is not None:
 			scaled_pixbuf = gtkgui_helpers.get_scaled_pixbuf(pixbuf, 'chat')
@@ -1650,6 +1680,9 @@ class ChatControl(ChatControlBase):
 					constants.KIND_CHAT_MSG_RECV):
 				kind = 'incoming'
 				name = self.contact.get_shown_name()
+			elif row[1] == constants.KIND_ERROR:
+				kind = 'status'
+				name = self.contact.get_shown_name()
 
 			tim = time.localtime(float(row[0]))
 
