@@ -13,7 +13,6 @@
 ##
 
 import os
-import sys
 import locale
 from common import gajim
 
@@ -98,11 +97,6 @@ class OptionsParser:
 	
 	def write(self):
 		(base_dir, filename) = os.path.split(self.__filename)
-		try:
-			base_dir = base_dir.decode(sys.getfilesystemencoding())
-			filename = filename.decode(sys.getfilesystemencoding())
-		except:
-			pass
 		self.__tempfile = os.path.join(base_dir, '.' + filename)
 		try:
 			f = open(self.__tempfile, 'w')
@@ -155,6 +149,18 @@ class OptionsParser:
 			self.update_config_to_01017()
 		if old < [0, 10, 1, 8] and new >= [0, 10, 1, 8]:
 			self.update_config_to_01018()
+		if old < [0, 11, 0, 1] and new >= [0, 11, 0, 1]:
+			self.update_config_to_01101()
+		if old < [0, 11, 0, 2] and new >= [0, 11, 0, 2]:
+			self.update_config_to_01102()
+		if old < [0, 11, 1, 1] and new >= [0, 11, 1, 1]:
+			self.update_config_to_01111()
+		if old < [0, 11, 1, 2] and new >= [0, 11, 1, 2]:
+			self.update_config_to_01112()
+		if old < [0, 11, 1, 3] and new >= [0, 11, 1, 3]:
+			self.update_config_to_01113()
+		if old < [0, 11, 1, 4] and new >= [0, 11, 1, 4]:
+			self.update_config_to_01114()
 
 		gajim.logger.init_vars()
 		gajim.config.set('version', new_version)
@@ -180,7 +186,7 @@ class OptionsParser:
 			'groupfontattrs', 'contacttextcolor', 'contactbgcolor', 'contactfont',
 			'contactfontattrs', 'bannertextcolor', 'bannerbgcolor', 'bannerfont', 
 			'bannerfontattrs']
-		for theme_name in (_('grocery'), _('gtk+')):
+		for theme_name in (_('grocery'), _('default')):
 			if theme_name not in gajim.config.get_per('themes'):
 				gajim.config.add_per('themes', theme_name)
 				theme = gajim.config.themes_default[theme_name]
@@ -211,7 +217,10 @@ class OptionsParser:
 
 	def assert_unread_msgs_table_exists(self):
 		'''create table unread_messages if there is no such table'''
-		con = sqlite.connect(logger.LOG_DB_PATH) 
+		back = os.getcwd()
+		os.chdir(logger.LOG_DB_FOLDER)
+		con = sqlite.connect(logger.LOG_DB_FILE)
+		os.chdir(back)
 		cur = con.cursor()
 		try:
 			cur.executescript(
@@ -280,7 +289,11 @@ class OptionsParser:
 
 	def update_config_to_01013(self):
 		'''create table transports_cache if there is no such table'''
-		con = sqlite.connect(logger.LOG_DB_PATH) 
+		#FIXME see #2812
+		back = os.getcwd()
+		os.chdir(logger.LOG_DB_FOLDER)
+		con = sqlite.connect(logger.LOG_DB_FILE)
+		os.chdir(back)
 		cur = con.cursor()
 		try:
 			cur.executescript(
@@ -299,8 +312,12 @@ class OptionsParser:
 
 	def update_config_to_01014(self):
 		'''apply indeces to the logs database'''
-		print _('migrating logs database to indeces')
-		con = sqlite.connect(logger.LOG_DB_PATH) 
+		print _('migrating logs database to indices')
+		#FIXME see #2812
+		back = os.getcwd()
+		os.chdir(logger.LOG_DB_FOLDER)
+		con = sqlite.connect(logger.LOG_DB_FILE)
+		os.chdir(back)
 		cur = con.cursor()
 		# apply indeces
 		try:
@@ -319,7 +336,11 @@ class OptionsParser:
 
 	def update_config_to_01015(self):
 		'''clean show values in logs database'''
-		con = sqlite.connect(logger.LOG_DB_PATH)
+		#FIXME see #2812
+		back = os.getcwd()
+		os.chdir(logger.LOG_DB_FOLDER)
+		con = sqlite.connect(logger.LOG_DB_FILE)
+		os.chdir(back)
 		cur = con.cursor()
 		status = dict((i[5:].lower(), logger.constants.__dict__[i]) for i in \
 			logger.constants.__dict__.keys() if i.startswith('SHOW_'))
@@ -347,12 +368,88 @@ class OptionsParser:
 		trayicon_notification_on_events '''
 		if self.old_values.has_key('trayicon_notification_on_new_messages'):
 			gajim.config.set('trayicon_notification_on_events',
-				 self.old_values['trayicon_notification_on_new_messages'])
+				self.old_values['trayicon_notification_on_new_messages'])
 		gajim.config.set('version', '0.10.1.7')
 
 	def update_config_to_01018(self):
 		'''chat_state_notifications -> outgoing_chat_state_notifications'''
 		if self.old_values.has_key('chat_state_notifications'):
 			gajim.config.set('outgoing_chat_state_notifications',
-				 self.old_values['chat_state_notifications'])
+				self.old_values['chat_state_notifications'])
 		gajim.config.set('version', '0.10.1.8')
+
+	def update_config_to_01101(self):
+		'''fill time_stamp from before_time and after_time'''
+		if self.old_values.has_key('before_time'):
+			gajim.config.set('time_stamp', '%s%%X%s ' % (
+				self.old_values['before_time'], self.old_values['after_time']))
+		gajim.config.set('version', '0.11.0.1')
+
+	def update_config_to_01102(self):
+		'''fill time_stamp from before_time and after_time'''
+		if self.old_values.has_key('ft_override_host_to_send'):
+			gajim.config.set('ft_add_hosts_to_send',
+				self.old_values['ft_override_host_to_send'])
+		gajim.config.set('version', '0.11.0.2')
+	
+	def update_config_to_01111(self):
+		'''always_hide_chatbuttons -> compact_view'''
+		if self.old_values.has_key('always_hide_groupchat_buttons') and \
+		self.old_values.has_key('always_hide_chat_buttons'):
+			gajim.config.set('compact_view', self.old_values['always_hide_groupchat_buttons'] and \
+			self.old_values['always_hide_chat_buttons'])
+		gajim.config.set('version', '0.11.1.1')
+
+	def update_config_to_01112(self):
+		'''gtk+ theme is renamed to default'''
+		if self.old_values.has_key('roster_theme') and \
+		self.old_values['roster_theme'] == 'gtk+':
+			gajim.config.set('roster_theme', _('default'))
+		gajim.config.set('version', '0.11.1.2')
+
+	def update_config_to_01113(self):
+		# copy&pasted from update_config_to_01013
+		back = os.getcwd()
+		os.chdir(logger.LOG_DB_FOLDER)
+		con = sqlite.connect(logger.LOG_DB_FILE)
+		os.chdir(back)
+		cur = con.cursor()
+		try:
+			cur.executescript(
+				'''
+				CREATE TABLE caps_cache (
+					node TEXT,
+					ver TEXT,
+					ext TEXT,
+					data BLOB
+				);
+				'''
+			)
+			con.commit()
+		except sqlite.OperationalError, e:
+			pass
+		con.close()
+		gajim.config.set('version', '0.11.1.3')
+
+	def update_config_to_01114(self):
+		# add default theme if it doesn't exist
+		d = ['accounttextcolor', 'accountbgcolor', 'accountfont',
+			'accountfontattrs', 'grouptextcolor', 'groupbgcolor', 'groupfont',
+			'groupfontattrs', 'contacttextcolor', 'contactbgcolor', 'contactfont',
+			'contactfontattrs', 'bannertextcolor', 'bannerbgcolor', 'bannerfont', 
+			'bannerfontattrs']
+		theme_name = _('default')
+		if theme_name not in gajim.config.get_per('themes'):
+			gajim.config.add_per('themes', theme_name)
+			if gajim.config.get_per('themes', 'gtk+'):
+				# copy from old gtk+ theme
+				for o in d:
+					val = gajim.config.get_per('themes', 'gtk+', o)
+					gajim.config.set_per('themes', theme_name, o, val)
+				gajim.config.del_per('themes', 'gtk+')
+			else:
+				# copy from default theme
+				theme = gajim.config.themes_default[theme_name]
+				for o in d:
+					gajim.config.set_per('themes', theme_name, o, theme[d.index(o)])
+		gajim.config.set('version', '0.11.1.4')
