@@ -63,6 +63,12 @@ def tree_cell_data_func(column, renderer, model, iter, tv=None):
 	# cell data func is global, because we don't want it to keep
 	# reference to GroupchatControl instance (self)
 	theme = gajim.config.get('roster_theme')
+	# allocate space for avatar only if needed
+	if isinstance(renderer, gtk.CellRendererPixbuf):
+		if model[iter][C_AVATAR]:
+			renderer.set_property('visible', True)
+		else:
+			renderer.set_property('visible', False)
 	if model.iter_parent(iter):
 		bgcolor = gajim.config.get_per('themes', theme, 'contactbgcolor')
 		if bgcolor:
@@ -97,13 +103,13 @@ def tree_cell_data_func(column, renderer, model, iter, tv=None):
 class PrivateChatControl(ChatControl):
 	TYPE_ID = message_control.TYPE_PM
 
-	def __init__(self, parent_win, contact, acct):
+	def __init__(self, parent_win, gc_contact, contact, acct):
 		room_jid = contact.jid.split('/')[0]
 		room_ctrl = gajim.interface.msg_win_mgr.get_control(room_jid, acct)
 		self.room_name = room_ctrl.name
+		self.gc_contact = gc_contact
 		ChatControl.__init__(self, parent_win, contact, acct)
 		self.TYPE_ID = 'pm'
-		self.display_names = (_('Private Chat'), _('Private Chats'))
 
 	def send_message(self, message):
 		'''call this function to send our message'''
@@ -138,11 +144,14 @@ class PrivateChatControl(ChatControl):
 
 class GroupchatControl(ChatControlBase):
 	TYPE_ID = message_control.TYPE_GC
+	# alphanum sorted
+	MUC_CMDS = ['ban', 'chat', 'query', 'clear', 'close', 'compact',
+		'help', 'invite', 'join', 'kick', 'leave', 'me', 'msg', 'nick',
+		'part', 'names', 'say', 'topic']
 
 	def __init__(self, parent_win, contact, acct):
 		ChatControlBase.__init__(self, self.TYPE_ID, parent_win,
-					'muc_child_vbox', (_('Group Chat'), _('Group Chats')),
-					contact, acct);
+					'muc_child_vbox', contact, acct);
 
 		widget = self.xml.get_widget('muc_window_actions_button')
 		id = widget.connect('clicked', self.on_actions_button_clicked)
@@ -180,24 +189,20 @@ class GroupchatControl(ChatControlBase):
 		self.nick = contact.name
 		self.name = self.room_jid.split('@')[0]
 
-		self.hide_chat_buttons_always = gajim.config.get(
+		hide_chat_buttons_always = gajim.config.get(
 			'always_hide_groupchat_buttons')
-		self.chat_buttons_set_visible(self.hide_chat_buttons_always)
+		self.chat_buttons_set_visible(hide_chat_buttons_always)
 		self.widget_set_visible(self.xml.get_widget('banner_eventbox'),
 			gajim.config.get('hide_groupchat_banner'))
 		self.widget_set_visible(self.xml.get_widget('list_scrolledwindow'),
 			gajim.config.get('hide_groupchat_occupants_list'))
-		self.gc_refer_to_nick_char = gajim.config.get('gc_refer_to_nick_char')
 
 		self._last_selected_contact = None # None or holds jid, account tuple
-		# alphanum sorted
-		self.muc_cmds = ['ban', 'chat', 'query', 'clear', 'close', 'compact',
-			'help', 'invite', 'join', 'kick', 'leave', 'me', 'msg', 'nick',
-			'part', 'names', 'say', 'topic']
+
 		# muc attention flag (when we are mentioned in a muc)
 		# if True, the room has mentioned us
 		self.attention_flag = False
-		self.room_creation = time.time()
+		self.room_creation = int(time.time()) # Use int to reduce mem usage
 		self.nick_hits = []
 		self.cmd_hits = []
 		self.last_key_tabs = False
@@ -210,10 +215,10 @@ class GroupchatControl(ChatControlBase):
 		# connect the menuitems to their respective functions
 		xm = gtkgui_helpers.get_glade('gc_control_popup_menu.glade')
 
-		widget = xm.get_widget('bookmark_room_menuitem')
-		id = widget.connect('activate',
+		self.bookmark_room_menuitem = xm.get_widget('bookmark_room_menuitem')
+		id = self.bookmark_room_menuitem.connect('activate',
 			self._on_bookmark_room_menuitem_activate)
-		self.handlers[id] = widget
+		self.handlers[id] = self.bookmark_room_menuitem
 
 		widget = xm.get_widget('change_nick_menuitem')
 		id = widget.connect('activate', self._on_change_nick_menuitem_activate)
@@ -272,14 +277,8 @@ class GroupchatControl(ChatControlBase):
 		# first one img, second one text, third is sec pixbuf
 		column = gtk.TreeViewColumn()
 
-		renderer_pixbuf = gtk.CellRendererPixbuf() # avatar image
-		column.pack_start(renderer_pixbuf, expand = False)
-		column.add_attribute(renderer_pixbuf, 'pixbuf', C_AVATAR)
-		column.set_cell_data_func(renderer_pixbuf, tree_cell_data_func,
-			self.list_treeview)
-		renderer_pixbuf.set_property('xalign', 1) # align pixbuf to the right
-
 		renderer_image = cell_renderer_image.CellRendererImage(0, 0) # status img
+		renderer_image.set_property('width', 26)
 		column.pack_start(renderer_image, expand = False)
 		column.add_attribute(renderer_image, 'image', C_IMG)
 		column.set_cell_data_func(renderer_image, tree_cell_data_func, 
@@ -288,9 +287,17 @@ class GroupchatControl(ChatControlBase):
 		renderer_text = gtk.CellRendererText() # nickname
 		column.pack_start(renderer_text, expand = True)
 		column.add_attribute(renderer_text, 'markup', C_TEXT)
+		renderer_text.set_property("ellipsize", pango.ELLIPSIZE_END)
 		column.set_cell_data_func(renderer_text, tree_cell_data_func,
 			self.list_treeview)
 
+		renderer_pixbuf = gtk.CellRendererPixbuf() # avatar image
+		column.pack_start(renderer_pixbuf, expand = False)
+		column.add_attribute(renderer_pixbuf, 'pixbuf', C_AVATAR)
+		column.set_cell_data_func(renderer_pixbuf, tree_cell_data_func,
+			self.list_treeview)
+		renderer_pixbuf.set_property('xalign', 1) # align pixbuf to the right
+
 		self.list_treeview.append_column(column)
 
 		# workaround to avoid gtk arrows to be shown
@@ -301,8 +308,9 @@ class GroupchatControl(ChatControlBase):
 		column.set_visible(False)
 		self.list_treeview.set_expander_column(column)
 
-		self.draw_banner()
-		self.got_disconnected() # init some variables
+		gajim.gc_connected[self.account][self.room_jid] = False
+		# disable win, we are not connected yet
+		ChatControlBase.got_disconnected(self) 
 
 		self.update_ui()
 		self.conv_textview.tv.grab_focus()
@@ -451,12 +459,10 @@ class GroupchatControl(ChatControlBase):
 		scaled_pix = pix.scale_simple(32, 32, gtk.gdk.INTERP_BILINEAR)
 		banner_status_img.set_from_pixbuf(scaled_pix)
 
-	def draw_banner(self):	
-		'''Draw the fat line at the top of the window that 
-		houses the muc icon, room jid, subject. 
+	def draw_banner_text(self):
+		'''Draw the text in the fat line at the top of the window that 
+		houses the room jid, subject. 
 		'''
-		ChatControlBase.draw_banner(self)
-
 		self.name_label.set_ellipsize(pango.ELLIPSIZE_END)
 		font_attrs, font_attrs_small = self.get_font_attrs()
 		text = '<span %s>%s</span>' % (font_attrs, self.room_jid)
@@ -477,6 +483,8 @@ class GroupchatControl(ChatControlBase):
 		childs = menu.get_children()
 		# hide chat buttons
 		childs[5].set_active(self.hide_chat_buttons_current)
+		if not gajim.connections[self.account].private_storage_supported:
+			self.bookmark_room_menuitem.set_sensitive(False)
 		if gajim.gc_connected[self.account][self.room_jid]:
 			c = gajim.contacts.get_gc_contact(self.account, self.room_jid,
 				self.nick)
@@ -500,7 +508,7 @@ class GroupchatControl(ChatControlBase):
 			else:
 				self.print_conversation(msg, nick, tim, xhtml)
 
-	def on_private_message(self, nick, msg, tim, xhtml):
+	def on_private_message(self, nick, msg, tim, xhtml, msg_id = None):
 		# Do we have a queue?
 		fjid = self.room_jid + '/' + nick
 		no_queue = len(gajim.events.get_events(self.account, fjid)) == 0
@@ -512,7 +520,7 @@ class GroupchatControl(ChatControlBase):
 			return
 
 		event = gajim.events.create_event('pm', (msg, '', 'incoming', tim,
-			False, '', None, xhtml))
+			False, '', msg_id, xhtml))
 		gajim.events.add_event(self.account, fjid, event)
 
 		autopopup = gajim.config.get('autopopup')
@@ -725,25 +733,33 @@ class GroupchatControl(ChatControlBase):
 
 	def set_subject(self, subject):
 		self.subject = subject
-		self.draw_banner()
+		self.draw_banner_text()
 
 	def got_connected(self):
 		gajim.gc_connected[self.account][self.room_jid] = True
 		ChatControlBase.got_connected(self)
-		self.draw_banner()
+		# We don't redraw the whole banner here, because only icon change
+		self._update_banner_state_image()
 
 	def got_disconnected(self):
 		self.list_treeview.get_model().clear()
 		nick_list = gajim.contacts.get_nick_list(self.account, self.room_jid)
 		for nick in nick_list:
+			# Update pm chat window
+			fjid = self.room_jid + '/' + nick
+			ctrl = gajim.interface.msg_win_mgr.get_control(fjid, self.account)
 			gc_contact = gajim.contacts.get_gc_contact(self.account, self.room_jid,
 				nick)
+			if ctrl:
+				gc_contact.show = 'offline'
+				gc_contact.status = ''
+				ctrl.update_ui()
+				ctrl.parent_win.redraw_tab(ctrl)
 			gajim.contacts.remove_gc_contact(self.account, gc_contact)
 		gajim.gc_connected[self.account][self.room_jid] = False
-		# Note, since this method is called during initialization it is NOT safe
-		# to call self.parent_win.redraw_tab here
 		ChatControlBase.got_disconnected(self)
-		self.draw_banner()
+		# We don't redraw the whole banner here, because only icon change
+		self._update_banner_state_image()
 
 	def draw_roster(self):
 		self.list_treeview.get_model().clear()
@@ -753,6 +769,8 @@ class GroupchatControl(ChatControlBase):
 			self.add_contact_to_roster(nick, gc_contact.show, gc_contact.role,
 						gc_contact.affiliation, gc_contact.status,
 						gc_contact.jid)
+		# Recalculate column width for ellipsizin
+		self.list_treeview.columns_autosize()
 
 	def on_send_pm(self, widget = None, model = None, iter = None, nick = None,
 	msg = None):
@@ -814,7 +832,7 @@ class GroupchatControl(ChatControlBase):
 		model[iter][C_AVATAR] = scaled_pixbuf
 
 	def chg_contact_status(self, nick, show, status, role, affiliation, jid,
-	reason, actor, statusCode, new_nick):
+	reason, actor, statusCode, new_nick, avatar_sha, tim = None):
 		'''When an occupant changes his or her status'''
 		if show == 'invisible':
 			return
@@ -823,10 +841,10 @@ class GroupchatControl(ChatControlBase):
 			role = 'visitor'
 		if not affiliation:
 			affiliation = 'none'
-
+		fake_jid = self.room_jid + '/' + nick
 		newly_created = False
 		if show in ('offline', 'error'):
-			if statusCode == '307':
+			if '307' in statusCode:
 				if actor is None: # do not print 'kicked by None'
 					s = _('%(nick)s has been kicked: %(reason)s') % {
 						'nick': nick,
@@ -837,7 +855,7 @@ class GroupchatControl(ChatControlBase):
 						'who': actor,
 						'reason': reason }
 				self.print_conversation(s, 'info')
-			elif statusCode == '301':
+			elif '301' in statusCode:
 				if actor is None: # do not print 'banned by None'
 					s = _('%(nick)s has been banned: %(reason)s') % {
 						'nick': nick,
@@ -848,7 +866,7 @@ class GroupchatControl(ChatControlBase):
 						'who': actor,
 						'reason': reason }
 				self.print_conversation(s, 'info')
-			elif statusCode == '303': # Someone changed his or her nick
+			elif '303' in statusCode: # Someone changed his or her nick
 				if nick == self.nick: # We changed our nick
 					self.nick = new_nick
 					s = _('You are now known as %s') % new_nick
@@ -878,17 +896,19 @@ class GroupchatControl(ChatControlBase):
 						os.path.join(path, puny_new_nick + ext)
 				for old_file in files:
 					if os.path.exists(old_file):
+						if os.path.exists(files[old_file]):
+							# Windows require this
+							os.remove(files[old_file])
 						os.rename(old_file, files[old_file])
 				self.print_conversation(s, 'info')
 
-			if len(gajim.events.get_events(self.account,
-			self.room_jid + '/' + nick)) == 0:
+			if len(gajim.events.get_events(self.account, fake_jid)) == 0:
 				self.remove_contact(nick)
 			else:
 				c = gajim.contacts.get_gc_contact(self.account, self.room_jid, nick)
 				c.show = show
 				c.status = status
-			if nick == self.nick and statusCode != '303': # We became offline
+			if nick == self.nick and '303' not in statusCode: # We became offline
 				self.got_disconnected()
 				self.parent_win.redraw_tab(self)
 		else:
@@ -897,28 +917,60 @@ class GroupchatControl(ChatControlBase):
 				iter = self.add_contact_to_roster(nick, show, role, affiliation,
 					status, jid)
 				newly_created = True
-				if statusCode == '201': # We just created the room
+				if '201' in statusCode: # We just created the room
 					gajim.connections[self.account].request_gc_config(self.room_jid)
 			else:
+				gc_c = gajim.contacts.get_gc_contact(self.account, self.room_jid,
+					nick)
+				# Re-get vcard if avatar has changed
+				# We do that here because we may request it to the real JID if we
+				# knows it. connections.py doesn't know it.
+				con = gajim.connections[self.account]
+				if gc_c.jid:
+					real_jid = gc_c.jid
+					if gc_c.resource:
+						real_jid += '/' + gc_c.resource
+				else:
+					real_jid = fake_jid
+				if con.vcard_shas.has_key(fake_jid):
+					if avatar_sha != con.vcard_shas[fake_jid]:
+						server = gajim.get_server_from_jid(self.room_jid)
+						if not server.startswith('irc'):
+							con.request_vcard(real_jid, fake_jid)
+				else:
+					cached_vcard = con.get_cached_vcard(fake_jid, True)
+					if cached_vcard and cached_vcard.has_key('PHOTO') and \
+					cached_vcard['PHOTO'].has_key('SHA'):
+						cached_sha = cached_vcard['PHOTO']['SHA']
+					else:
+						cached_sha = ''
+					if cached_sha != avatar_sha:
+						# avatar has been updated
+						# sha in mem will be updated later
+						server = gajim.get_server_from_jid(self.room_jid)
+						if not server.startswith('irc'):
+							con.request_vcard(real_jid, fake_jid)
+					else:
+						# save sha in mem NOW
+						con.vcard_shas[fake_jid] = avatar_sha
+
 				actual_role = self.get_role(nick)
 				if role != actual_role:
 					self.remove_contact(nick)
 					self.add_contact_to_roster(nick, show, role,
 						affiliation, status, jid)
 				else:
-					c = gajim.contacts.get_gc_contact(self.account, self.room_jid,
-						nick)
-					if c.show == show and c.status == status and \
-						c.affiliation == affiliation: #no change
+					if gc_c.show == show and gc_c.status == status and \
+						gc_c.affiliation == affiliation: # no change
 						return
-					c.show = show
-					c.affiliation = affiliation
-					c.status = status
+					gc_c.show = show
+					gc_c.affiliation = affiliation
+					gc_c.status = status
 					self.draw_contact(nick)
 
 		self.parent_win.redraw_tab(self)
 		if (time.time() - self.room_creation) > 30 and \
-				nick != self.nick and statusCode != '303':
+				nick != self.nick and '303' not in statusCode:
 			st = ''
 			print_status = None
 			for bookmark in gajim.connections[self.account].bookmarks:
@@ -932,7 +984,8 @@ class GroupchatControl(ChatControlBase):
 				# delete ressource
 				simple_jid = gajim.get_jid_without_resource(jid)
 				nick_jid += ' (%s)' % simple_jid
-			if show == 'offline' and print_status in ('all', 'in_and_out'):
+			if show == 'offline' and print_status in ('all', 'in_and_out') and \
+			'307' not in statusCode:
 				st = _('%s has left') % nick_jid
 				if reason:
 					st += ' [%s]' % reason
@@ -979,10 +1032,16 @@ class GroupchatControl(ChatControlBase):
 		server = gajim.get_server_from_jid(self.room_jid)
 		if gajim.config.get('ask_avatars_on_startup') and \
 		not server.startswith('irc'):
-			fjid = self.room_jid + '/' + nick
-			pixbuf = gtkgui_helpers.get_avatar_pixbuf_from_cache(fjid, True)
+			fake_jid = self.room_jid + '/' + nick
+			pixbuf = gtkgui_helpers.get_avatar_pixbuf_from_cache(fake_jid, True)
 			if pixbuf == 'ask':
-				gajim.connections[self.account].request_vcard(fjid, True)
+				if j:
+					fjid = j
+					if resource:
+						fjid += '/' + resource
+					gajim.connections[self.account].request_vcard(fjid, fake_jid)
+				else:
+					gajim.connections[self.account].request_vcard(fake_jid, fake_jid)
 		if nick == self.nick: # we became online
 			self.got_connected()
 		self.list_treeview.expand_row((model.get_path(role_iter)), False)
@@ -1039,7 +1098,14 @@ class GroupchatControl(ChatControlBase):
 			# example: /nick foo
 			if len(message_array) and message_array[0] != self.nick:
 				nick = message_array[0]
-				gajim.connections[self.account].change_gc_nick(self.room_jid, nick)
+				try:
+					nick = helpers.parse_resource(nick)
+				except:
+					# Invalid Nickname
+					dialogs.ErrorDialog(_('Invalid nickname'),
+					_('The nickname has not allowed characters.'))
+					return True
+				gajim.connections[self.account].join_gc(nick, self.room_jid, None)
 				self.clear(self.msg_textview)
 			else:
 				self.get_command_help(command)
@@ -1102,19 +1168,13 @@ class GroupchatControl(ChatControlBase):
 			if len(message_array):
 				message_array = message_array[0].split()
 				invitee = message_array.pop(0)
-				if invitee.find('@') >= 0:
-					reason = ' '.join(message_array)
-					gajim.connections[self.account].send_invite(self.room_jid,
-						invitee, reason)
-					s = _('Invited %(contact_jid)s to %(room_jid)s.') % {
-						'contact_jid': invitee,
-						'room_jid': self.room_jid}
-					self.print_conversation(s, 'info')
-					self.clear(self.msg_textview)
-				else:
-					#%s is something the user wrote but it is not a jid so we inform
-					s = _('%s does not appear to be a valid JID') % invitee
-					self.print_conversation(s, 'info')
+				reason = ' '.join(message_array)
+				gajim.connections[self.account].send_invite(self.room_jid, invitee, reason)
+				s = _('Invited %(contact_jid)s to %(room_jid)s.') % {
+					'contact_jid': invitee,
+					'room_jid': self.room_jid}
+				self.print_conversation(s, 'info')
+				self.clear(self.msg_textview)
 			else:
 				self.get_command_help(command)
 			return True
@@ -1247,7 +1307,8 @@ class GroupchatControl(ChatControlBase):
 
 	def get_command_help(self, command):
 		if command == 'help':
-			self.print_conversation(_('Commands: %s') % self.muc_cmds, 'info')
+			self.print_conversation(_('Commands: %s') % GroupchatControl.MUC_CMDS,
+				'info')
 		elif command == 'ban':
 			s = _('Usage: /%s <nickname|JID> [reason], bans the JID from the group chat.'
 				' The nickname of an occupant may be substituted, but not if it '
@@ -1257,7 +1318,7 @@ class GroupchatControl(ChatControlBase):
 			self.print_conversation(s, 'info')
 		elif command == 'chat' or command == 'query':
 			self.print_conversation(_('Usage: /%s <nickname>, opens a private chat'
-				' window to the specified occupant.') % command, 'info')
+				' window with the specified occupant.') % command, 'info')
 		elif command == 'clear':
 			self.print_conversation(
 				_('Usage: /%s, clears the text window.') % command, 'info')
@@ -1286,7 +1347,7 @@ class GroupchatControl(ChatControlBase):
 				(command, command), 'info')
 		elif command == 'msg':
 			s = _('Usage: /%s <nickname> [message], opens a private message window'
-				'and sends message to the occupant specified by nickname.') % \
+				' and sends message to the occupant specified by nickname.') % \
 				command
 			self.print_conversation(s, 'info')
 		elif command == 'nick':
@@ -1318,14 +1379,31 @@ class GroupchatControl(ChatControlBase):
 		'''asks user for new nick and on ok it sets it on room'''
 		def on_ok(widget):
 			nick = instance.input_entry.get_text().decode('utf-8')
-			gajim.connections[self.account].change_gc_nick(self.room_jid, nick)
-			self.nick = nick
+			try:
+				nick = helpers.parse_resource(nick)
+			except:
+				# invalid char
+				dialogs.ErrorDialog(_('Invalid nickname'),
+				_('The nickname has not allowed characters.'))
+				return
+			gajim.connections[self.account].join_gc(nick, self.room_jid, None)
 		instance = dialogs.InputDialog(title, prompt, proposed_nick,
 			is_modal = False, ok_handler = on_ok)
 
 	def shutdown(self, status='offline'):
 		gajim.connections[self.account].send_gc_status(self.nick, self.room_jid,
 							show='offline', status=status)
+		nick_list = gajim.contacts.get_nick_list(self.account, self.room_jid)
+		for nick in nick_list:
+			# Update pm chat window
+			fjid = self.room_jid + '/' + nick
+			ctrl = gajim.interface.msg_win_mgr.get_control(fjid, self.account)
+			if ctrl:
+				contact = gajim.contacts.get_gc_contact(self.account, self.room_jid, nick)
+				contact.show = 'offline'
+				contact.status = ''
+				ctrl.update_ui()
+				ctrl.parent_win.redraw_tab(ctrl)
 		# They can already be removed by the destroy function
 		if self.room_jid in gajim.contacts.get_gc_list(self.account):
 			gajim.contacts.remove_room(self.account, self.room_jid)
@@ -1338,6 +1416,8 @@ class GroupchatControl(ChatControlBase):
 			if self.handlers[i].handler_is_connected(i):
 				self.handlers[i].disconnect(i)
 			del self.handlers[i]
+		# Remove unread events from systray
+		gajim.events.remove_events(self.account, self.room_jid)
 
 	def allow_shutdown(self, method):
 		'''If check_selection is True, '''
@@ -1467,8 +1547,8 @@ class GroupchatControl(ChatControlBase):
 			splitted_text2 = text.split(None, 1)
 			if text.startswith('/topic '):
 				if len(splitted_text2) == 2 and \
-					    self.subject.startswith(splitted_text2[1]) and\
-					    len(self.subject) > len(splitted_text2[1]):
+				self.subject.startswith(splitted_text2[1]) and\
+				len(self.subject) > len(splitted_text2[1]):
 					message_buffer.insert_at_cursor(
 						self.subject[len(splitted_text2[1]):])
 					return True
@@ -1481,7 +1561,7 @@ class GroupchatControl(ChatControlBase):
 			if text.startswith('/') and len(splitted_text) == 1:
 				text = splitted_text[0]
 				if len(text) == 1: # user wants to cycle all commands
-					self.cmd_hits = self.muc_cmds
+					self.cmd_hits = GroupchatControl.MUC_CMDS
 				else:
 					# cycle possible commands depending on what the user typed
 					if self.last_key_tabs and len(self.cmd_hits) and \
@@ -1490,7 +1570,7 @@ class GroupchatControl(ChatControlBase):
 						self.cmd_hits.pop(0)
 					else: # find possible commands
 						self.cmd_hits = []
-						for cmd in self.muc_cmds:
+						for cmd in GroupchatControl.MUC_CMDS:
 							if cmd.startswith(text.lstrip('/')):
 								self.cmd_hits.append(cmd)
 				if len(self.cmd_hits):
@@ -1506,9 +1586,10 @@ class GroupchatControl(ChatControlBase):
 			else:
 				begin = ''
 
+			gc_refer_to_nick_char = gajim.config.get('gc_refer_to_nick_char')
 			if len(self.nick_hits) and \
 					self.nick_hits[0].startswith(begin.replace(
-					self.gc_refer_to_nick_char, '')) and \
+					gc_refer_to_nick_char, '')) and \
 					self.last_key_tabs: # we should cycle
 				self.nick_hits.append(self.nick_hits[0])
 				self.nick_hits.pop(0)
@@ -1516,13 +1597,14 @@ class GroupchatControl(ChatControlBase):
 				self.nick_hits = [] # clear the hit list
 				list_nick = gajim.contacts.get_nick_list(self.account,
 									self.room_jid)
+				list_nick.remove(self.nick) # Skip self
 				for nick in list_nick:
 					if nick.lower().startswith(begin.lower()):
 						# the word is the begining of a nick
 						self.nick_hits.append(nick)
 			if len(self.nick_hits):
 				if len(splitted_text)  < 2: # This is the 1st word of the line or no word
-					add = self.gc_refer_to_nick_char + ' '
+					add = gc_refer_to_nick_char + ' '
 				else:
 					add = ' '
 				start_iter = end_iter.copy()
@@ -1681,12 +1763,11 @@ class GroupchatControl(ChatControlBase):
 
 	def _start_private_message(self, nick):
 		gc_c = gajim.contacts.get_gc_contact(self.account, self.room_jid, nick)
-		c = gajim.contacts.contact_from_gc_contact(gc_c)
-		nick_jid = c.jid
+		nick_jid = gc_c.get_full_jid()
 
 		win = gajim.interface.msg_win_mgr.get_window(nick_jid, self.account)
 		if not win:
-			gajim.interface.roster.new_chat(c, self.account, private_chat = True)
+			gajim.interface.roster.new_private_chat(gc_c, self.account)
 			win = gajim.interface.msg_win_mgr.get_window(nick_jid, self.account)
 		win.set_active_tab(nick_jid, self.account)
 		win.window.present()
@@ -1775,7 +1856,8 @@ class GroupchatControl(ChatControlBase):
 				start = ' '
 			add = ' '
 		else:
-			add = self.gc_refer_to_nick_char + ' '
+			gc_refer_to_nick_char = gajim.config.get('gc_refer_to_nick_char')
+			add = gc_refer_to_nick_char + ' '
 		message_buffer.insert_at_cursor(start + nick + add)
 
 	def on_list_treeview_motion_notify_event(self, widget, event):
