@@ -103,13 +103,13 @@ class MessageWindow:
 			self.notebook.set_show_tabs(False)
 		self.notebook.set_show_border(gajim.config.get('tabs_border'))
 
-		# set up DnD
-		self.hid = self.notebook.connect('drag_data_received',
-			self.on_tab_label_drag_data_received_cb)
-		self.handlers[self.hid] = self.notebook
-		
-		self.notebook.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.DND_TARGETS,
-			gtk.gdk.ACTION_MOVE)
+		# set up DnD if GTK+ version < 2.10, use OUR way to reorder tabs
+		if gtk.pygtk_version < (2, 10, 0) or gtk.gtk_version < (2, 10, 0):
+			self.hid = self.notebook.connect('drag_data_received',
+				self.on_tab_label_drag_data_received_cb)
+			self.handlers[self.hid] = self.notebook
+			self.notebook.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.DND_TARGETS,
+				gtk.gdk.ACTION_MOVE)
 
 	def change_account_name(self, old_name, new_name):
 		if self._controls.has_key(old_name):
@@ -191,7 +191,11 @@ class MessageWindow:
 		control.handlers[id] = tab_label_box
 		self.notebook.append_page(control.widget, tab_label_box)
 
-		self.setup_tab_dnd(control.widget)
+		# If GTK+ version >= 2.10, use gtk native way to reorder tabs
+		if gtk.pygtk_version >= (2, 10, 0) and gtk.gtk_version >= (2, 10, 0):
+			self.notebook.set_tab_reorderable(control.widget, True)
+		else:
+			self.setup_tab_dnd(control.widget)
 
 		self.redraw_tab(control)
 		self.window.show_all()
@@ -235,6 +239,11 @@ class MessageWindow:
 
 	def show_title(self, urgent = True, control = None):
 		'''redraw the window's title'''
+		if not control:
+			control = self.get_active_control()
+		if not control:
+			# No more control in this window
+			return
 		unread = 0
 		for ctrl in self.controls():
 			if ctrl.type_id == message_control.TYPE_GC and not \
@@ -253,8 +262,6 @@ class MessageWindow:
 		else:
 			urgent = False
 
-		if not control:
-			control = self.get_active_control()
 		if control.type_id == message_control.TYPE_GC:
 			name = control.room_jid.split('@')[0]
 			urgent = control.attention_flag
@@ -264,11 +271,17 @@ class MessageWindow:
 				name += '/' + control.resource
 
 		window_mode = gajim.interface.msg_win_mgr.mode
+
 		if self.get_num_controls() == 1:
 			label = name
 		elif window_mode == MessageWindowMgr.ONE_MSG_WINDOW_PERTYPE:
 			# Show the plural form since number of tabs > 1
-			label = control.display_names[1]
+			if self.type == 'chat':
+				label = _('Chats')
+			elif self.type == 'gc':
+				label = _('Group Chats')
+			else:
+				label = _('Private Chats')
 		else:
 			label = _('Messages')
 		title = _('%s - Gajim') % label
@@ -288,11 +301,11 @@ class MessageWindow:
 		ctrl_page = self.notebook.page_num(ctrl.widget)
 		self.notebook.set_current_page(ctrl_page)
 	
-	def remove_tab(self, ctrl, mothod, reason = None, force = False):
+	def remove_tab(self, ctrl, method, reason = None, force = False):
 		'''reason is only for gc (offline status message)
 		if force is True, do not ask any confirmation'''
 		# Shutdown the MessageControl
-		if not force and not ctrl.allow_shutdown(mothod):
+		if not force and not ctrl.allow_shutdown(method):
 			return
 		if reason is not None: # We are leaving gc with a status message
 			ctrl.shutdown(reason)
@@ -304,7 +317,10 @@ class MessageWindow:
 			types = ['printed_msg', 'chat', 'gc_msg'])
 		del gajim.last_message_time[ctrl.account][ctrl.get_full_jid()]
 
-		self.disconnect_tab_dnd(ctrl.widget)
+		# Disconnect tab DnD only if GTK version < 2.10
+		if gtk.pygtk_version < (2, 10, 0) or gtk.gtk_version < (2, 10, 0):
+			self.disconnect_tab_dnd(ctrl.widget)
+
 		self.notebook.remove_page(self.notebook.page_num(ctrl.widget))
 
 		fjid = ctrl.get_full_jid()
@@ -537,10 +553,10 @@ class MessageWindow:
 		'''Set tab label as drag source and connect the drag_data_get signal'''
 		tab_label = self.notebook.get_tab_label(child)
 		tab_label.dnd_handler = tab_label.connect('drag_data_get', 
-							  self.on_tab_label_drag_data_get_cb)
+			self.on_tab_label_drag_data_get_cb)
 		self.handlers[tab_label.dnd_handler] = tab_label
 		tab_label.drag_source_set(gtk.gdk.BUTTON1_MASK, self.DND_TARGETS,
-					 gtk.gdk.ACTION_MOVE)
+			gtk.gdk.ACTION_MOVE)
 		tab_label.page_num = self.notebook.page_num(child)
 
 	def on_tab_label_drag_data_get_cb(self, widget, drag_context, selection,
@@ -573,14 +589,14 @@ class MessageWindow:
 			tab_alloc = tab.get_allocation()
 			if horiz:
 				if (x >= tab_alloc.x) and \
-					   (x <= (tab_alloc.x + tab_alloc.width)):
+				(x <= (tab_alloc.x + tab_alloc.width)):
 					page_num = i
 					if x >= tab_alloc.x + (tab_alloc.width / 2.0):
 						to_right = True
 					break
 			else:
 				if (y >= tab_alloc.y) and \
-					   (y <= (tab_alloc.y + tab_alloc.height)):
+				(y <= (tab_alloc.y + tab_alloc.height)):
 					page_num = i
 				
 					if y > tab_alloc.y + (tab_alloc.height / 2.0):
@@ -621,10 +637,10 @@ class MessageWindowMgr:
 
 	def __init__(self):
 		''' A dictionary of windows; the key depends on the config:
-		 ONE_MSG_WINDOW_NEVER: The key is the contact JID
-		 ONE_MSG_WINDOW_ALWAYS: The key is MessageWindowMgr.MAIN_WIN 
-		 ONE_MSG_WINDOW_PERACCT: The key is the account name
-		 ONE_MSG_WINDOW_PERTYPE: The key is a message type constant'''
+		ONE_MSG_WINDOW_NEVER: The key is the contact JID
+		ONE_MSG_WINDOW_ALWAYS: The key is MessageWindowMgr.MAIN_WIN 
+		ONE_MSG_WINDOW_PERACCT: The key is the account name
+		ONE_MSG_WINDOW_PERTYPE: The key is a message type constant'''
 		self._windows = {}
 		# Map the mode to a int constant for frequent compares
 		mode = gajim.config.get('one_message_window')
